/*
  Copyright 2011-2016 Marvell Semiconductor, Inc.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

      http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
*/
import Pins from "pins";

var main = {
	state: 1,
	changeable: true,
	remote: undefined, //Stores connection to other lamp when available
	hold: 2000, //Defines what is considered a 'long touch.' Here it's 2000ms.
	timer: undefined,
	touchState: 0,
	shareName: "element-lamps-demo",
	onLaunch() {
		Pins.configure({	
			TouchSensor: {
				require: "Digital",
				pins:{
					power: { pin: 9, type: "Power" },
					digital: { pin: 10, direction: "input" },
					ground: { pin: 11, type: "Ground" },
					ground2: { pin: 15, type: "Ground" },
				}
			},
			Relay: {
				require:"Digital",
				pins:{
					ground: { pin: 12, type: "Ground" },
					power: { pin: 13, type: "Power" },
					digital: { pin:14, direction: "ouput" },	
				}
			},
		}, success=> {
			Pins.invoke("/Relay/write", this.state);
			Pins.share({ type: "ws" }, { zeroconf: true, name: this.shareName });
			Pins.discover(connectionDescription => {
				trace("onFound: " + JSON.stringify(connectionDescription) + "\n");
				if(connectionDescription.name.indexOf(this.shareName) != -1){
						this.remote = Pins.connect(connectionDescription);
				}
			}, connectionDescription => {
				trace("onLost" + JSON.stringify(connectionDescription) + "\n");
				if(connectionDescription.name.indexOf(this.shareName) != -1){
					if (this.remote) this.remote.close();
					this.remote = undefined;
				}
			});
			this.startReading();
		});
	},
	startReading() {
		Pins.repeat("/TouchSensor/read", 100, result => {
			if (result == 1){
				if(!this.touchState){
					this.touchState = 1;
				}
				if(!this.timer)
					this.timer = setTimeout(()=>{					
						if (this.remote) this.remote.invoke("/Relay/write", this.state);
					}, this.hold);

				if(this.changeable) {
					this.changeable = false;

					// update the current status
					Pins.invoke("/Relay/read", result=>{
						this.state = parseInt(result);
						this.state = this.state ? 0 : 1;
						Pins.invoke("/Relay/write",  this.state);
					});
				}
			}
			else {
				if(this.touchState == 1){
					this.touchState = 0;
				}
				if(this.timer)
					clearTimeout(this.timer);
				this.timer = undefined;	
				this.changeable = true;
			}
		});
	},
	onQuit() {
		Pins.share();
		if (this.remote) {
			this.remote.close();
			this.remote = undefined;
		}
	}
}

export default main;